/*****************************************************************************************************
 *********************************** TPS929120 *******************************************************
 *****************************************************************************************************/

#include "TPS929120.h"

volatile uint8_t bufreceive[3], cRX=0;  //definicja globalnej zmienne potrzebnej podczaso odbieirania oraz nadawania danych
volatile uint32_t tpstimer;

//Wartosci domyslne przepisywane do EEPROM poczawszy od rejestru EEPM0 (0xC0)
const uint8_t generaldefault[16][2]={{0xc0, 0x00}, {0xC1, 0x00}, {0xC2, 0xff}, {0xc3, 0x0f}, {0xc4, 0x00},
{0xc5, 0x00}, {0xc6, 0x00}, {0xc7, 0xa3}, {0xc8, 0x03}, {0xc9, 0x00}, {0xca, 0x00}, {0xcb, 0x00},
{0xcc, 0x00}, {0xcd, 0x00}, {0xce, 0x00}, {0xcf, 0x00}};

//Funkcja delay - argumenty podajemy w ms
void delaymsTPS(uint32_t ms)
{
	tpstimer=ms*10;
	while(tpstimer);
}

//Funkcja wysylajaca wiadomosci w postaci zmiennej lancuchowej
void SendBytesViaUART( uint8_t *wsktab, uint8_t amoundbytes)
{
	uint8_t *s;

	s=wsktab;
	for(uint8_t i=0; i<amoundbytes; i++)
	{
		while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET)
		{

		}
		    	USART_SendData(USART1, (uint8_t)*s);
				s++;
	}
	while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET); //odczekanie az bufor do wysylania bedzie pusty
}

//byteval - wskaznik na tablice zawierajaca dane do obliczenia CRC, numberofbyte - ilosc bajtow w tablicy z ktorych bedzie liczone CRC, generator - np b100011101=0x11D -> 0x1D (najstarszy bit pomijamy)
//initvalue - warto pomijana, check crc=0 jesli obliczamy CRC natomiast !=0  gdy zamierzamy sprawdzac dan warto
uint8_t Calculate_CRC8(uint8_t *byteVal, uint8_t numbersofbyte, uint8_t generator, uint8_t initvalue, uint8_t checkcrc)
{
    uint8_t crc=0x00; // poczatkowa inicjalizacja zmiennej do obliczania CRC
    uint8_t *k;
    uint8_t buffor[numbersofbyte+1];
    uint8_t p;

    k=byteVal;

    for(p=0; p<numbersofbyte; p++)
    {
    	buffor[p]=*k;
    	k++;
    }

    if(checkcrc)  //jesli sprawdzamy wartosc do wpisujemy ostatnio obliczona wartosc CRC
    {
    	 buffor[p]=checkcrc;
    }
    else buffor[p]=0x00;  //dopisanie 8 bitow 0 na samym koncu szeregu bajtow poniewaz wilomian(generator) jest stopnia 8

    k=buffor;

     if(initvalue) //wartosc XOR
         {
         	*k=(*k) ^ initvalue;
         }

    //wykonanie instrukcji dla kazdego bajtu w tym take bajtu+1 (1 poniewaz mamy bajt inicjalizacyjny)
   for(uint8_t s=0; s<=numbersofbyte; s++)
    {
        for (int i = 7; i >= 0; i--)  //zmienna int aby wartosci moglby byc ujemne oraz zeby warunek mogl byc spelniony
        {
            // sprawdzenie czy najstarszy bajt jest rny od zera
            if ((crc & 0x80) != 0)
            {
                crc = (uint8_t)(crc << 1);

                if(((*k & (1 << i)) != 0))
					{
						crc=(uint8_t)(crc | 0x01);
					}
					else
					{
						crc=(uint8_t)(crc & 0xFE);
					}
                //jezeli najstarszy bit byl 1 to nastepuje jego pominiecie oraz pomniniecie najstarszego bitu generatora oraz utworzenie XOR dla aktualnego crc oraz generatora
                crc = (uint8_t)(crc ^ generator);
            }
            else
            {   //jezeli nie jest ustawiony najstaszy bajt - nastepuje po prostu przesuniecie aktualnego crc
                crc = (uint8_t)(crc << 1);
                if(((*k & (1 << i)) != 0))
                {
                	crc=(uint8_t)(crc | 0x01);
                }
                else
                	{
                	crc=(uint8_t)(crc & 0xFE);
                	}
            }
        }
        k++; //inkrementacja bajtu
    }
    return crc;
}


//Odwrocenie ciagu bitow (lustrzane odbicie) sluzace jako np. reflect input lub mozna takze reflect output
uint8_t reflectforcrc(uint8_t *inputtable, uint8_t *outputtable, uint8_t amoundbytes)
{
	uint32_t resoult=0;
	uint8_t *k, *l;

	k=inputtable;
	l=outputtable;


	for(int i=(amoundbytes-1); i>=0; i--)
	{
		    for(int z=7; z>=0; z--)
			{
				if(*k & (1<<z)) resoult |= (1<<(7-z));
			}
		    *l=resoult;
		    resoult=0;
		    k++;
		    l++;
	}
	return 1;
}


//funkcja ktora wpisuje lub odbiera dana z tps929120, DEV_ADDR - zawiera adres wraz z najstaszym bietem okreslajacym czy wpisujemy czy
//odbieramy dane z rejestru, REG_ADDR - zawiera adres rejestru, DATA_N - zwiera dana, wskRX - zawiera ustawiony wskaznik na zmienna globalna
//ktora sprawdza bufor odbiorczy, jezeli odebrano dwa bajty wtedy wartosc wskaznika wynosi 2 i nastapilo odebranie danej + CRC z drivera
//wskbuf - zawiera wskaznik na bufor odbiorczy w ktorym sa przechowywane dane odebrane. Dane odbierane powinny byc w przerwaniach
uint8_t TPS929120(uint8_t DEV_ADDR, uint8_t REG_ADDR, uint8_t DATA_N, volatile uint8_t *wskRX,volatile uint8_t *wskbuf)
{
		uint8_t temporarybytes[5]; //zmienna tymczasowa
		uint8_t respondcrc=0;
		uint8_t receiveddata=0;
		volatile uint8_t *tempwskbuf;
		uint32_t dec;


		tempwskbuf=wskbuf;

		if(DEV_ADDR & 0x80) //wysylanie danych do TPS929120
		{
		temporarybytes[0]=DEV_ADDR;
		temporarybytes[1]=REG_ADDR;
		temporarybytes[2]=DATA_N;
		reflectforcrc(temporarybytes, temporarybytes, 3);

		temporarybytes[4]=Calculate_CRC8(temporarybytes, 3, 0x31, 0xff, 0); //przepisanie bajtu CRC do wysylanej tablicy
		temporarybytes[0]=0x55; //wpisanie bajtu synchronizujacego
		temporarybytes[1]=DEV_ADDR;
		temporarybytes[2]=REG_ADDR;
		temporarybytes[3]=DATA_N;


		*wskRX=0; //wykasowanie wskaznika na dane odebrane
		SendBytesViaUART( temporarybytes, 5); //wyslanie bajtow
		}
		else
		{
		 temporarybytes[0]=DEV_ADDR;
		 temporarybytes[1]=REG_ADDR;
		 reflectforcrc(temporarybytes, temporarybytes, 2);

		 temporarybytes[3]=Calculate_CRC8(temporarybytes, 2, 0x31, 0xff, 0); //przepisanie bajtu CRC do wysylanej tablicy
		 temporarybytes[0]=0x55; //wpisanie bajtu synchronizujacego
		 temporarybytes[1]=DEV_ADDR;
		 temporarybytes[2]=REG_ADDR;
		 temporarybytes[4]=0x00;


		*wskRX=0; //wykasowanie wskaznika na dane odebrane
		 SendBytesViaUART( temporarybytes, 4); //wyslanie bajtow
		}

		dec=10000000;  //deklaracja zmiennej aby sprawdzic czy czasem tps sie nie zawiesil i nie nastapilo zawieszenie programu
		//dec=10000;
		while((*wskRX<2)  && dec)
		{
			 asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP");
			 asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP"); asm("NOP");
			 dec--;
		}

		if(!dec) return 0; //zwroc 0 jezeli uplynal czas na odbior danej

		*wskRX=0;
		temporarybytes[0]=*tempwskbuf;
		tempwskbuf++;
		respondcrc=*tempwskbuf;

		receiveddata=temporarybytes[0];

		reflectforcrc(temporarybytes, temporarybytes, 1);  //dane wejsciowe jako reflect


		if(Calculate_CRC8(temporarybytes, 1, 0x31, 0xff, respondcrc)) //sprawdzenie czy suma kontrolna jest rowna 0 czy jeden z danej odebranej
		{
			//jesli jest rozna od zera wtedy zwroc wartosc 0 - blad przy odbieraniu danej
			return 0;
		}
		else
		{
			//jesli suma kontrolna sie zgadza czyli jest rowna 0 wtedy zwroc odebrana wartosc
			return receiveddata;
		}

}

//Funkcja wpisujaca dane do okreslonego rejestru drivera TPS929120
void Write_data_TPS(uint8_t adress, uint8_t regadress, uint8_t data)
{
	TPS929120(adress | WRITE_MASK, regadress, data, &cRX, bufreceive);
}

//Funkcja odczytujace dane z okreslonego rejestru drivera TPS929120
uint8_t Read_data_TPS(uint8_t adress, uint8_t regadress)
{
	return TPS929120(adress & READ_MASK, regadress, 0x00, &cRX, bufreceive);
}

//odblokowanie rejestrow TPS929120
void Unlock_TPS(uint8_t adress)
{
	Write_data_TPS(adress, 0x61, 0x00);
}

//zablokowanie dostepu do rejestrow
void Lock_TPS(uint8_t adress)
{
	Write_data_TPS(adress, 0x61, 0x0f);
}


//funkcja za pomoca ktorej mozemy wylaczac lub wlaczac dane wyjscie TPS - adress podajemy adrss drivera, outnumber - numer wyjscia 0-11, enablesiable
//nalezy wpisac definicje ENABLE_TPS_OUT, DISABLE_TPS_OUT
void Control_Output_TPS(uint8_t adress, uint8_t outnumber, uint8_t enabledisable)
{
	uint8_t temp;
	uint8_t outtemp;
	uint8_t regadress;


	if(outnumber<8)
		{
		outtemp=outnumber;
		regadress=0x50;
		}
	else
		{
		outtemp=outnumber-8;
		regadress=0x51;
		}

		temp=Read_data_TPS(adress, regadress);
		if(enabledisable)
		{
			outtemp=temp | (1<<outtemp);
		}
		else
		{
			outtemp=temp & (~(1<<outtemp));
		}


		Write_data_TPS(adress, regadress, outtemp);
}

//ustawienie bitu w danym rejestrze
void Set_Bit_TPS(uint8_t adress, uint8_t regadress, uint8_t bitnumber)
{
	uint8_t temp=Read_data_TPS(adress, regadress);
	Write_data_TPS(adress, regadress,temp | (1<<bitnumber));
}

//wykasowanie bitu w danym rejestrze
void Clear_Bit_TPS(uint8_t adress, uint8_t regadress, uint8_t bitnumber)
{
	uint8_t temp=Read_data_TPS(adress, regadress);
	Write_data_TPS(adress, regadress,temp & (~(1<<bitnumber)));
}

//wpisywanie danych do rejestrow obiektowo
void TPS_Setup(uint8_t adress, ConNormalMode *param)
{
	 Write_data_TPS(adress, ICONFBASEADDR + (param->OUT), param->ICONF);
	 Write_data_TPS(adress, PWMBASEADDR + (param->OUT), param->PWM);
}

//Funkcja za pomoca ktorej mozna zmienic wszystkie 12 wyjsc jednoczesnie
void WriteOutTPS(uint8_t adress, uint16_t outputs)
{
	Write_data_TPS(adress, 0x50, (uint8_t)outputs);
	Write_data_TPS(adress, 0x51, (uint8_t)(outputs>>8));
}

//Funkcja za pomoca ktrej mozemy odblokowac moliwosc zapisania do pamieci eeprom ustawien
void Unlock_EEPROM_TPS(uint8_t adress)
{

	//sekwencja odblokowania pamieci eeprom przy wykorzystywaniu zewnetrznego adresu
	Write_data_TPS(adress, CONF_EEPGATE, 0x00);
	Write_data_TPS(adress, CONF_EEPGATE, 0x02);
	Write_data_TPS(adress, CONF_EEPGATE, 0x01);
	Write_data_TPS(adress, CONF_EEPGATE, 0x09);
	Write_data_TPS(adress, CONF_EEPGATE, 0x02);
	Write_data_TPS(adress, CONF_EEPGATE, 0x09);

	Write_data_TPS(adress, CONF_EEPMODE, 0x01);
	Write_data_TPS(adress, CONF_STAYINEEP, 0x80);

}

//Funkcja ktora aktywuje uruchomienie danych z shadowEEPROM do EEPROM
void Write_EEPROM_TPS(uint8_t adress)
{
	Write_data_TPS(adress, CONF_EEPPROG,  0x04);

	delaymsTPS(500);
}

//Funkcja przechodzaca do normalnego trybu pracy i zamykajaca eeprom
void Lock_EEPROM_TPS(uint8_t adress)
{

	Write_data_TPS(adress, CONF_STAYINEEP, 0x00);
}

//Funkcja za pomoca ktorej nastpuje wpisanie wartosci do pamieci eeprom do fail state 1 oraz fail0 oraz fail state1
void EEPROM_FAIL_STATE(uint8_t adress, uint16_t FS0, uint16_t FS1)
{
	Write_data_TPS(adress, 0xC0, (uint8_t)FS0);
	Write_data_TPS(adress, 0xC1, (uint8_t)(FS0>>8));
	Write_data_TPS(adress, 0xC2, (uint8_t)FS1);
	Write_data_TPS(adress, 0xC3, (uint8_t)(FS1>>8));
}

//Funkcja za pomoca ktorej zmona ustawiac odpowiednie wartosci w pamieci EEPROM
void TPS_EEP_Setup(uint8_t adress, ConFSMode *param)
{
	Write_data_TPS(adress, EEPIBASE + (param->OUT), param->EEPI);
	Write_data_TPS(adress, EEPPBASE + (param->OUT), param->EEPP);
}

//Funkcja ktora wymusza przejscie do trybu Fail-State
void TPS_Force_FS(uint8_t adress)
{
	uint8_t temp=Read_data_TPS(adress, 0x60);
	temp&=~(1<<2);
	Write_data_TPS(adress, 0x60, temp| 0x20);
}

//Funkcja ktora powoduje powrot ze stanu Fail State do stanu normalnego
void TPS_Return_FS(uint8_t adress)
{
	uint8_t temp;
	Unlock_TPS(adress);
    temp=Read_data_TPS(adress, 0x60);
    temp&=~(0x20);
	Write_data_TPS(adress, 0x60, temp | 0x4);
}

//Funkcja odlbokowujaca EEPROM za pomoca REF i przywracajaca mu wartosci domysle dla rejestrow ustawien
void Default_EEPROM_TPS_BY_REF(uint8_t adress)
{

	//W tym miejscu nalezy podciagnac za pomoca portu pin REF drivera w zaleznosci od wybranego drivera poprzez adress
	//Pin RED mozna podciagnac recznie przed aktywowaniem funkcji, natomiast przed wywolanie jej musi TPS nalezy odblokowac

	//sekwencja przywrocenia wartosci domyslnych pamieci eeprom przy wykorzystywaniu wewnetrznego adresu
	Write_data_TPS(0x00, CONF_EEPGATE, 0x09);
	Write_data_TPS(0x00, CONF_EEPGATE, 0x02);
	Write_data_TPS(0x00, CONF_EEPGATE, 0x09);
	Write_data_TPS(0x00, CONF_EEPGATE, 0x01);
	Write_data_TPS(0x00, CONF_EEPGATE, 0x02);
	Write_data_TPS(0x00, CONF_EEPGATE, 0x00);

	Write_data_TPS(0x00, CONF_EEPMODE, 0x01);
	Write_data_TPS(0x00, CONF_STAYINEEP, 0x80);

	for(int k=0; k<16; k++)
	{
		 Write_data_TPS(0x00, generaldefault[k][0], generaldefault[k][1]);
	}
	Write_data_TPS(0x00, 0x64,  0x04);
	delaymsTPS(500);

}

//Przywrocenie wartosci domyslnych za pomoca programowania poprzez ADDR
void Default_EEPROM_TPS_BY_ADDR(uint8_t adress)
{

	//sekwencja przywrocneia wartosci domyslnych eeprom przy wykorzystywaniu zewnetrznego adresu
	Write_data_TPS(adress, CONF_EEPGATE, 0x00);
	Write_data_TPS(adress, CONF_EEPGATE, 0x02);
	Write_data_TPS(adress, CONF_EEPGATE, 0x01);
	Write_data_TPS(adress, CONF_EEPGATE, 0x09);
	Write_data_TPS(adress, CONF_EEPGATE, 0x02);
	Write_data_TPS(adress, CONF_EEPGATE, 0x09);

	Write_data_TPS(adress, CONF_EEPMODE, 0x01);
	Write_data_TPS(adress, CONF_STAYINEEP, 0x80);

	for(int k=0; k<16; k++)
	{
		 Write_data_TPS(adress, generaldefault[k][0], generaldefault[k][1]);
	}
	Write_data_TPS(adress, 0x64,  0x04);
	delaymsTPS(500);
	Write_data_TPS(adress, CONF_STAYINEEP, 0x00);

}

//Funkcja obliczajaca CRC pamieci EEPROM i wpisujaca go do ostatniego rejestru aby fail-state dzialal prawidlowo
//Przed wywolanie funkcji nalezy wgrac nowe wartosci do pamieci eeprom (wybrane rejestry) a nastepnie
//nalezy wywolac ponizsza funkcje, ktora zmienia ostatni rejestrs
void Write_CRC_EEPROM(uint8_t adress)
{
    uint8_t valueeeprom[39];

	uint8_t crcvalue;

	Read_ALL_EEPROM(adress,valueeeprom);  //odczytanie wszystkich rejestrow eeprom oproc rejestru CRC

	reflectforcrc(valueeeprom, valueeeprom, 39);   //obliczenie refleks
	crcvalue=Calculate_CRC8(valueeeprom, 39, 0x31, 0xff, 0); //obliczenie wartosci CRC

	Unlock_EEPROM_TPS(adress); //odblokowanie pamieci eeprom
	Write_data_TPS(adress, 0xCF, crcvalue);

	Write_EEPROM_TPS(adress); //uruchomienie zapisu do pamieci EEPROM
	delaymsTPS(500);
	Lock_EEPROM_TPS(adress); //zamkniecie pramieci eeprom

}

//Odczytanie wartosci eeprom register
void Read_ALL_EEPROM(uint8_t adress,uint8_t *data)
{
	uint8_t *z;
	z=data;

	for(int k=0x80; k<=0x8B; k++)
			{
				*z=Read_data_TPS(adress, k);
				z++;
			}

		for(int k=0xA0; k<=0xAB; k++)
			{
				*z=Read_data_TPS(adress, k);
				z++;
			}

		for(int k=0xC0; k<=0xCE; k++)
			{
				*z=Read_data_TPS(adress, k);
				z++;
			}

}

//Funkcja zwracajaca wartosc z przetwornika dla okreslonego kanalu
uint8_t getADC(uint8_t adress, uint8_t channel)
{
	uint8_t val=0;
	uint32_t guard=0;

	if(((channel>5) && (channel<0x10)) || (channel>0x1B)) return 0;

	Write_data_TPS(adress, 0x62, channel); //wybranie kanalu pomiaru oraz uruchomienie konwersji
	guard=10;

	while(!val && guard)
	{
		val=Read_data_TPS(adress, 0x71);
		val=val&0x04;
		guard--;
	}

	if(!guard) return 0;

	return Read_data_TPS(adress, 0x73);
}

//Funkcja boliczajaca wartosc z przetwornicka ADC - wartosc np. 825 oznacza 8.25V. Wyjatek dla temperatury wartos np 132 odznacza stopnie celcjusza zlacza
long int calcvalueADC(uint8_t adress, uint8_t channel)
{
	long int value;
	uint32_t a, k;
	const uint32_t factors[7][2]={ {70, 101}, {673, 804}, {465, 220}, {1, 2152}, {7592, 7461}, {673, 804}, {673, 804}};

    if(((channel>5) && (channel<0x10)) || (channel>0x1B)) return 0;

    if(channel<6)
    {
    	a=factors[channel][0];
    	k=factors[channel][1];
    }
    else
    {
    	a=factors[6][0];
    	k=factors[6][1];
    }

    value=getADC(adress, channel);
    if(channel==3)
    {
    	value=value*k;
    	value=value/1000;
    	value=value-242;
    }
    else
    {
	value=value*k;
	value=value+a;
	value=value/100;
    }

	return value;
}

//Funkcja ktora odczytuje wszystkie rejestry statusowe
FlagStruct ReadFlags(uint8_t adress)
{
	FlagStruct temp;
	uint8_t *wsk, ad=0x70;

	wsk=(uint8_t *)&temp;

	for(uint8_t i=0; i<6; i++) //inkrementacja adresu
	{
		*wsk=Read_data_TPS(adress, ad);
		wsk++;
		ad++;
	}

	ad=0x77;
	for(uint8_t i=0; i<2; i++) //inkrementacja adresu
		{
			*wsk=Read_data_TPS(adress, ad);
			wsk++;
			ad++;
		}
	ad=0x7B;
	for(uint8_t i=0; i<4; i++) //inkrementacja adresu
		{
				*wsk=Read_data_TPS(adress, ad);
				wsk++;
				ad++;
		}
	return temp;
}


//Funkcja za pomoca mozna odczytac okreslony rejestr konfiguracyjny
uint8_t ReadConfRegister(uint8_t adress, ConfigRegisters confregister)
{
	return Read_data_TPS(adress, confregister);
}

//Funkcja kasujac licznik WDT gdy jest zalaczony, zwraca wartosc rejestru STATUS
uint8_t WDT_Reset_TPS(uint8_t adress)
{
	return Read_data_TPS(adress, 0x70);
}

//Funkcja za pomoca, ktorej wpisujemy wartosci konfiguracyjne
//Najpierw nalezy zdefiniowac strukture ConfStruct a nastepnie odniesc sie do niej &
void Write_Conf_TPS(uint8_t adress, ConfStruct *wsk)
{
	uint8_t temporary;

	ConfigRegisters ADDR;
	ADDR=CONF_EN0;
	Write_data_TPS(adress, ADDR, (uint8_t)(wsk->CONF_ENCHx));
	ADDR=CONF_EN1;
	Write_data_TPS(adress, ADDR, (uint8_t)((wsk->CONF_ENCHx)>>8));
	ADDR=CONF_DIAGEN0;
	Write_data_TPS(adress, ADDR, (uint8_t)(wsk->CONF_DIAGCHx));
	ADDR=CONF_DIAGEN1;
	Write_data_TPS(adress, ADDR, (uint8_t)((wsk->CONF_DIAGCHx)>>8));
	ADDR=CONF_MISC0;
	temporary=(((wsk->CONF_AUTOSSx)<<7) | ((wsk->CONF_LDOx)<<6) | ((wsk->CONF_EXPx)<<4));
	Write_data_TPS(adress, ADDR, temporary);
	ADDR=CONF_MISC1;
	temporary=((wsk->CONF_PWMFREQ)<<4);
	temporary|=(wsk->CONF_REFRANGEx);
	Write_data_TPS(adress, ADDR, temporary);
	ADDR=CONF_MISC2;
	temporary=((wsk->CONF_FLTIMEOUT)<<4);
	temporary|=(wsk->CONF_ADCLOWSUPTH);
	Write_data_TPS(adress, ADDR, temporary);
	ADDR=CONF_MISC3;
	temporary=((wsk->CONF_ODIOUT)<<4);
	temporary|=(wsk->CONF_ODPW);
	Write_data_TPS(adress, ADDR, temporary);
	ADDR=CONF_MISC4;
	Write_data_TPS(adress, ADDR, (uint8_t)((wsk->CONF_WDTIMER)<<4));
}

//Funkcja ktora wpisuje wartosci do rejestru MISC7
void Write_Misc_7_TPS(uint8_t adress, Misc_7_Struct *wsk)
{
	ConfigRegisters ADDR;
	uint8_t temporary;
	ADDR=CONF_MISC7;

	temporary=((wsk->CONF_EXTCLK)<<5) | ((wsk->CONF_READSHADOW)<<1) | (((wsk->CONF_SHAREPWM)<<4));
	Write_data_TPS(adress, ADDR, temporary);

}

//Funkcja za pomoca ktorej mozemy zamaskowac dzialanie wyjscia EER
void ERR_Conf_Mask(uint8_t adress, ERR_Structure_Mask *wsk)
{
	ConfigRegisters ADDR=CONF_MISC8;
	uint8_t temporary;

	temporary=((wsk->CONF_MASKTSD)<<3) | ((wsk->CONF_MASKOPEN)<<4) | ((wsk->CONF_MASKSHORT)<<5) | ((wsk->CONF_MASKCRC)<<6) | ((wsk->CONF_MASKREF)<<7);

	Write_data_TPS(adress, ADDR, temporary);
}

//Funkcja kasujaca flagi FAUL lub POR
//@arg value -> FAULT_CLR | POR_CLR
void CLR_write(uint8_t adress, uint8_t value)
{
	uint8_t temp=Read_data_TPS(adress, 0x60);
	Write_data_TPS(adress, 0x60, value | temp);
}

//Wysterowanie wyjscia ERR stan niski przez 50ms - open colector - automatycznie powraca do 0 w normal state
void PullDownERR(uint8_t adress)
{
	uint8_t temp=Read_data_TPS(adress, 0x60);
	Write_data_TPS(adress, 0x60, temp | ERR_PULL_DOWN);
}


