#include "stm32f10x.h"

//----------------------------------------------------------------------------------------------------
#define debugmode 0     //zalaczenie modelu debugowania dla odczytu wartosci eeprom
#define defaultvalueref 0 //po uruchomieniu tej definicji jest mozliwosc zresetowania ukladu eeprom do wartosci domyslnych przy zwarciu ref do 5V
//-----------------------------------------------------------------------------------------------------
//-----------------------------  Definicje ------------------------------------------------------------
#define infoOFF GPIOC->BRR = GPIO_Pin_3
#define infoON GPIOC->BSRR = GPIO_Pin_3
#define dcdcregoff GPIOC->BRR = GPIO_Pin_0
#define dcdcregon GPIOC->BSRR = GPIO_Pin_0

#define divider 4
#define  clearwdt  IWDG_ReloadCounter()


//-----------------------------------------------------------------------------------------------------
//--------------------------- FUNKCJE SYSTEMOWE -------------------------------------------------------
void GPIO_Config(void);
void NVIC_Config(void);
void USART_Config(void);
void RCC_Config(void);
void DMA_Config(void);
void ADC_Config(void);
void DAC_Config(void);
void TIM_Config(void);
void IWDT_config(void);

//-----------------------------------------------------------------------------------------------------
//--------------------------- FUNKCJE UYTKOWE --------------------------------------------------------
void reulationDCDC(uint16_t *tab1, uint16_t *tab2, uint8_t amoundstairs, uint16_t valadc, uint8_t state);
uint16_t avadcfunction(uint16_t actualvalue);
void EEPchangeTPS(void);
void returnfromFS(void);
void checkFSandreturn(void);
void signalprogramLED(uint16_t time);
void Flash_program(void);
void Flash_read(void);
void checkchanges(void);
//-----------------------------------------------------------------------------------------------------
//-------------------------- ZMIENNE ------------------------------------------------------------------
volatile uint8_t flagawd;
volatile uint32_t TIMERUS, timertt, timerregdac, timadc, timercheck, timprog1, timerprog2;
extern volatile uint8_t bufreceive[3], cRX;
uint16_t valueADC, maxADC, valdiag1, valdiag2, valdiag3, averageadc;
uint8_t stateofbutton=0, directionofbutton, animationpwrled=3, powerledpower=100, animationstairs=4, amoundofstairs=24, red=0x00, blue=0xff, green=0xff, tresh, addtime,  val1, val2,
	    activationdiagnostic, clrflag, powertps=0;
extern volatile uint16_t uart2rx;
extern volatile unsigned char btbuff[300];
uint16_t valuesvoltages[72], openloadtable[72], shortdiagtable[72];
uint32_t  *kwsk;

uint8_t debugtable[40];


//******************************************************
//Definicje dla obslugi pamieci FLASH

uint32_t EraseCounter = 0x00, Address = 0x00;
uint32_t NbrOfPage = 0x00;
volatile FLASH_Status FLASHStatus=FLASH_COMPLETE;

#define OffsetNazwy 0xA000
#define BaseAdresFlash 0x08000000

#define BANK1_WRITE_START_ADDR  ((uint32_t)0x0800A000)
#define BANK1_WRITE_END_ADDR    ((uint32_t)0x0800A400)
#define FLASH_PAGE_SIZE         ((uint16_t)0x400)
//*******************************************************


int main(void)
{

	RCC_Config();
	NVIC_Config();
	GPIO_Config();
	USART_Config();
	ADC_Config();
    DMA_Config();
    TIM_Config();
    DAC_Config();

    infoON; //uruchomienie diody informujacej o przeprowadzaniu inicjalizacji systemu - w tym czasie nie nalezy nic robic

	    //Inicjalizacja zegrar systemowego systick
		if (SysTick_Config_Mod(SysTick_CLKSource_HCLK_Div8, 900)) //przerwanie co 0,1ms
		{
			while(1);		// W razie bledu petla nieskonczona
		}

		//Uruchomienie przetwornica ADC
		ADC_SoftwareStartConvCmd(ADC1, ENABLE);

		//odczekanie az napiecie zasilajace tps bedzie odpowiednio duze
		while(powertps>550)
		{
		powertps=avadcfunction(valueADC);
		}

		//odblokowanie dostpu do wszystkich TPS po uruchomieniu
		for(uint8_t s=0; s<6; s++)
		{
			Unlock_TPS(s);
			TIMERUS=100;
			while(TIMERUS); //opoznienie zeby odblokowac wszystkie tps
		}

		//Funkcje ktore musza byc wykonane tylko jeden raz podczas pierwszego uruchomienia
		kwsk=(u32 *)(BaseAdresFlash + OffsetNazwy);  //ustawienie wskaznika na adres komorki ktora jest flaga czy juz byl dokonany zapis i pierwsze uruchonmienie


		if(*kwsk!=0x234fab21) //sprawdzenie po zalaczeniu czy pamiec juz byla uzywana i nie jest to pierwszy raz po zapisie
		{
			 setupBT(); //inicjalizacja modulu BT
			 EEPchangeTPS(); //zaprogramowanie funkcji eeprom

		     signalprogramLED(200); //funkcja informujaca dioda led o rozpoczeciu programowaniu
			 Flash_program();
	         returnfromFS(); //poworot z trybu FS jezeli uklad znajduje sie w takim trybie

	         signalprogramLED(500); //funkcja informujaca dioda led o rozpoczeciu programowaniu
		}

		Flash_read();

#if defaultvalueref==1
	    for(uint8_t q=0; q<6; q++)
	    {
		Default_EEPROM_TPS_BY_REF(q);
	    }
#endif
#if debugmode==1
		Read_ALL_EEPROM_CRC(0,debugtable);
#endif


		returnfromFS(); //poworot z trybu FS jezeli uklad znajduje sie w takim trybie

	enablediagnostic(); //sprawdzenie czy wszystkie diagnostyki sa zalaczone dla wszystkich driverow

	IWDT_config(); //inicjalizacja licznika watchdog przed wejscie do gwnej petli programu
	dcdcregoff;
	infoOFF; //wylaczenie diody inicjalizacyjnej mowiace o tym, e system wchodzi do glownej petli programu
	//--------------------------------------------------------------------------------------------------------------------------------------
	while(1) //Gwna ptla programu
	{
		//Aplikacje
		detectbutton(&stateofbutton, &directionofbutton); //funkcja okreslajaca stan przyciskow oraz kierunek
		powerled(powerledpower, directionofbutton, stateofbutton, animationpwrled); //obsluga diod power led
		stairsanimation( directionofbutton, stateofbutton, animationstairs, amoundofstairs, tresh, addtime); //funkcja animacyjna
		resetwatchdogtps(); //reset licznika wdt
		changecolorstairs(red, green, blue, amoundofstairs);  //funkcja zmieniajaca kolory diod led
		clearwdt; //wykasowanie watchdog systemowego
		//Obsluga BT
		seekBT((unsigned char *)btbuff, (uint16_t)uart2rx, &val1, &val2); //Przeszukanie bufora odbiorczego
		setsettingsBT(&val1, &val2, &animationpwrled, &animationstairs, &powerledpower, &amoundofstairs, &red, &green, &blue, &tresh, &addtime, &stateofbutton, &directionofbutton, &activationdiagnostic); //wykonanie zmian ustawien w zaleznosci od bufora odbiorczego
		clearwdt; //wykasowanie watchdog systemowego
		//Funkcje diagnostyczne
		valdiag1=maxvoltage(amoundofstairs, valuesvoltages, stateofbutton); //pomiar napiec dla wszystkich kanalach, po zmierzeniu wszystkich wyjsc funkcja zwraca wartosc maksymalna
		valdiag2=(uint8_t)openload(amoundofstairs, openloadtable, stateofbutton); //sprawdzenie open load na wszystkich kanalach
		valdiag3=(uint8_t)shortdiag(amoundofstairs, shortdiagtable, stateofbutton); //sprawdzenie zwarc na wszystkich kanalach
		BTsenddiagnostic(&activationdiagnostic, valdiag1, valdiag2, valdiag3, valuesvoltages, openloadtable, shortdiagtable, amoundofstairs); //funkcja wysylajaca wszystkie dane
		clrflag=clearfaultflag(&valdiag1, &valdiag2, &valdiag3, amoundofstairs); //wykasowanie wszystkich flag z driverow
		clearwdt; //wykasowanie watchdog systemowego
		//Regulacja napiecia z przetwornicy
		averageadc=avadcfunction(valueADC); //wartosc srednia
		reulationDCDC(valuesvoltages, openloadtable, amoundofstairs, averageadc, stateofbutton); //funkcja dokonujaca reulacji na podstawie przeprowadzonego pomiaru
		clearwdt; //wykasowanie watchdog systemowego

		//Funkcja zapisujaca do pamieci nowe ustawienia
		checkchanges();
	}
	//Koniec glowne petli
	//--------------------------------------------------------------------------------------------------------------------------------------
}

void USART_Config(void)
{

	  //Konfiguracja USART1
	    USART_InitTypeDef  USART_InitStructure;

	    //Wpisywanie ilosci bodow
	    USART_InitStructure.USART_BaudRate =115200;
	    USART_InitStructure.USART_WordLength = USART_WordLength_8b;                     //Dlugosc slowa = 8 bitow
	    USART_InitStructure.USART_StopBits = USART_StopBits_1;                          //1 bit stopu
	    USART_InitStructure.USART_Parity = USART_Parity_No;                             //Brak kontroli parzystosci
	    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None; //Kontrola przeplywu danych (brak)
	    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;	                //Tryb pracy

	    USART_Init(USART1, &USART_InitStructure);
	    USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
	    USART_Cmd(USART1, ENABLE);  //zalaczenie konfiguracji


	    USART_InitStructure.USART_BaudRate =9600;

	    USART_InitStructure.USART_WordLength = USART_WordLength_8b;                     //Dlugosc slowa = 8 bitow
	    USART_InitStructure.USART_StopBits = USART_StopBits_1;                          //1 bit stopu
	    USART_InitStructure.USART_Parity = USART_Parity_No;                             //Brak kontroli parzystosci
	    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None; //Kontrola przeplywu danych (brak)
	    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;	                //Tryb pracy
	    USART_Init(USART3, &USART_InitStructure);
	    USART_ITConfig(USART3, USART_IT_RXNE, ENABLE);  	                       	//zalaczenie przerwania od USART

	    USART_Cmd(USART3, ENABLE);  //zalaczenie konfiguracji

}

void NVIC_Config(void)
{
	NVIC_InitTypeDef   NVIC_InitStructure;

	NVIC_InitStructure.NVIC_IRQChannel =SysTick_IRQn ; //tutaj trzeba zwrocic uwage na definicje w pliu stm32f10x.h
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);

  	NVIC_InitStructure.NVIC_IRQChannel =USART1_IRQn ; //tutaj trzeba zwrocic uwage na definicje w pliu stm32f10x.h
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);

  	//Definicje priorytetu oraz podpriorytetu dla przerwania od USART3 - komunikacja BT
  	NVIC_InitStructure.NVIC_IRQChannel =USART3_IRQn ; //tutaj trzeba zwrocic uwage na definicje w pliu stm32f10x.h
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 3;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
 	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);
}

void GPIO_Config(void)
{
  //konfigurowanie portow GPIO
   GPIO_InitTypeDef  GPIO_InitStructure;

  //UART1 - podlaczony do driverow LED
  //PA9 - Tx
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	//PA10 - Rx
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);


     //Konwiguracja portow dla USART3
    //PB10 - Tx
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
 	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
 	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
 	GPIO_Init(GPIOB, &GPIO_InitStructure);

 	//PB11 - Rx
 	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
 	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
 	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
 	GPIO_Init(GPIOB, &GPIO_InitStructure);

 	//Konfiguracja wejscia przetwornika jako wejscia analogowego - ADC
 	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
 	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
 	GPIO_Init(GPIOA, &GPIO_InitStructure);

	//Konfiguracja wyjsc dla przetwornika DAC
    GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_4;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

     //Wyjscie dla ukladow PWM
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP; //wyjscie alternatywne o wysokiej impedancji
    GPIO_Init(GPIOB, &GPIO_InitStructure);

      //Porty wyjsciowe
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOC, &GPIO_InitStructure);

    //Obsluga portow wejsciowych
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    dcdcregoff;
}


void RCC_Config(void)
//konfigurowanie sygnalow taktujacych
{

  RCC_DeInit();                                          //Reset ustawien RCC
  RCC_HSEConfig(RCC_HSE_ON);                             //Wlaczenie HSE


	RCC_HCLKConfig(RCC_SYSCLK_Div1);                     //ustaw HCLK=SYSCLK
	RCC_PCLK2Config(RCC_HCLK_Div4);                      //ustaw PCLK2=HCLK
	RCC_PCLK1Config(RCC_HCLK_Div4);                      //ustaw PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_PREDIV1, RCC_PLLMul_9); //ustaw PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz


    RCC_PLLCmd(ENABLE);                                  //wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET){}  //odczekaj na poprawne uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           //ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08){}//odczekaj az PLL bedzie sygnalem zegarowym systemu

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE); //zalaczenie wyjsc alternatywnych

         /*kod zwiazany z konfiguracja sygnalow zegarowych potrzebnych w programie peryferiow*/
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);

		RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);//wlacz taktowanie USART1
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);//wlaczenie taktowania dla USART3

		//Zalaczenie szyn odpowiedzialnych za pomiary wartosci
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE); //wlacz taktowanie ADC1
		//Podzia czestotliwosci dla ADC
		RCC_ADCCLKConfig(RCC_PCLK2_Div8);

		RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);   //wlacz taktowanie DMA
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_DAC, ENABLE);  //zalaczenie przetwornika DAC
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE); //wlacz taktowanie TIM4

}

void DMA_Config(void)
{
  //konfigurowanie DMA

  #define ADC1_DR_Address 0x4001244C;                                             //adres rejestru ADC1->DR

  DMA_InitTypeDef DMA_InitStructure;

  DMA_DeInit(DMA1_Channel1);                                                      //Usun ewentualna poprzednia konfiguracje DMA

  DMA_InitStructure.DMA_PeripheralBaseAddr = (unsigned long int)ADC1_DR_Address;  //Adres docelowy transferu
  DMA_InitStructure.DMA_MemoryBaseAddr = (unsigned long int)&valueADC;            //Adres poczatku bloku do przeslania
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;                              //Kierunek transferu
  DMA_InitStructure.DMA_BufferSize = 1;                                           //Liczba elementow do przeslania (dlugosc bufora)
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;                //Wylaczenie automatycznego zwiekszania adresu po stronie ADC
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;                         //Wylaczenie automatycznego zwiekszania adresu po stronie pamieci (bufora)
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;     //Rozmiar pojedynczych przesylanych danych po stronie ADC (HalfWord = 16bit)
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;             //Rozmiar pojedynczych przesylanych danych po stronie pamieci (bufora)
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;                                 //Tryb dzialania kontrolera DMA - powtarzanie cykliczne
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;                             //Priorytet DMA - wysoki
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;                                    //Wylaczenie obslugi transferu z pamieci do pamieci
  DMA_Init(DMA1_Channel1, &DMA_InitStructure);                                    //Zapis konfiguracji

  //Wlacz DMA, kanal 1
  DMA_Cmd(DMA1_Channel1, ENABLE);
}

//Konfiguracja ADC
void ADC_Config(void)
{
  //konfigurowanie przetwornika AC
  ADC_InitTypeDef ADC_InitStructure;

	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;                  //Jeden przetwornik, praca niezalezna
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;                        //Pomiar jednego kanalu, brak skanowania kanalow
    ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;                  //Pomiar w trybie ciaglym
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;	//Brak wyzwalania zewnetrznego
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;            	//Wyrownanie danych do prawej - 12 mlodszych bitow znaczacych
	ADC_InitStructure.ADC_NbrOfChannel = 1; 	                          //Liczba uzywanych kanalow =1
	ADC_Init(ADC1, &ADC_InitStructure);                                 //Incjalizacja przetwornika

    ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_239Cycles5);
                                                                      //Kanal 1 - GPIOA1
                                                                      //Grupa podstawowa, czas probkowania 41,5+12,5 cykli (calkowity czas przetwarania)
   ADC_DMACmd(ADC1,ENABLE);                                            //Wlaczenie DMA dla ADC
   ADC_Cmd(ADC1, ENABLE);	                                            //Wlacz ADC1

   ADC_ResetCalibration(ADC1);	                                        //Reset rejestrow kalibracyjnych ADC1
   while(ADC_GetResetCalibrationStatus(ADC1));	                        //Odczekanie na wykonanie resetu
   ADC_StartCalibration(ADC1);	                                        //Kalibracja ADC1
   while(ADC_GetCalibrationStatus(ADC1));    	                        //Odczekanie na zakonczenie kalibracji ADC1
}

void DAC_Config(void)
{
  DAC_InitTypeDef   DAC_InitStructure;

  DAC_DeInit();

  DAC_InitStructure.DAC_Trigger = DAC_Trigger_None;							//Wyzwalanie ciagle
  DAC_InitStructure.DAC_WaveGeneration = DAC_WaveGeneration_None;			//Wylaczenie generacji fsli
  DAC_InitStructure.DAC_OutputBuffer = DAC_OutputBuffer_Enable;				//Zalaczenie buffora wyjsciowego

  DAC_Init(DAC_Channel_1, &DAC_InitStructure);							    //inicjalizacja

  DAC_Cmd(DAC_Channel_1, ENABLE);											//zalaczenie
}


void TIM_Config(void)
{
  //konfigurowanie licznikow

  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;

  // Konfiguracja TIM3
  // Ustawienia ukladu podstawy czasu
  TIM_TimeBaseStructure.TIM_Prescaler = 600;         //bez prescelera (prescaler=1)
  TIM_TimeBaseStructure.TIM_Period = 1000ul;      //
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV4;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);
  TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);


  // Konfiguracja kanalu 1, 2, 3, 4 - uzywamy kanalu 3 poniewaz jego wyjscie jest na GPIOB0
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1; //tryb pwm
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 0ul;          //poczatkowe wypelnienie maksymalne (TIMpulse/Timperiod)*100%=100%
  TIM_OCInitStructure.TIM_OCPolarity=TIM_OCPolarity_High;
  TIM_OC3Init(TIM4, &TIM_OCInitStructure);
  TIM_OC2Init(TIM4, &TIM_OCInitStructure);
  TIM_OC1Init(TIM4, &TIM_OCInitStructure);


  TIM_OC3PreloadConfig(TIM4, TIM_OCPreload_Enable);//wlaczenie buforowania
  TIM_OC2PreloadConfig(TIM4, TIM_OCPreload_Enable);//wlaczenie buforowania
  TIM_OC1PreloadConfig(TIM4, TIM_OCPreload_Enable);//wlaczenie buforowania


  TIM_ARRPreloadConfig(TIM4, ENABLE);//wlaczenie buforowania
  // Wlaczenie timera
  TIM_Cmd(TIM4, ENABLE);
  //ustawienie wartosci na 0 po uruchoeminiu
  (TIM4->CCR1)=0;
  (TIM4->CCR2)=0;
  (TIM4->CCR3)=0;
}

//Procedura inicjalizacyjna licznik watchdog
void IWDT_config(void)
{
	IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);  //zezwolenie na zapis do rejestrow IWDG

	IWDG_SetPrescaler(IWDG_Prescaler_256); // ustawienie preskalera IWDT na 256 czyli 40kHz/256=156Hz
	IWDG_SetReload(0x270);      //ustawienie wartosci przeladowania na okolo 3s
	IWDG_ReloadCounter();       //przeladowanie IWDT
	IWDG_Enable();              //zalaczenie iwdt
}

//Funkcja regulacyjna napiecie z przetwornicy DCDC
void reulationDCDC(uint16_t *tab1, uint16_t *tab2, uint8_t amoundstairs, uint16_t valadc, uint8_t state)
{
	static uint8_t firstuse=1;
	static long int valueofdac=0xfff;
	static uint16_t tempvalue;
	uint16_t *wsk1, *wsk2;
	uint16_t comp;

	if(firstuse)
	{
		firstuse=0; //usuniecie flagi
		DAC_SetChannel1Data(DAC_Align_12b_R, (uint16_t)valueofdac); //aktywowanie funkcji dac
		dcdcregon;  //zalaczenie regulacji
	}

	if(!timerregdac && state)
	{
		tempvalue=0;  //wykasowanie aktualnej wartosci
		for(uint8_t t=0; t<(amoundstairs*3); t++)
		{
			wsk1=tab1+t;
			wsk2=tab2+t;

			//if(*wsk1)
			if(*wsk1 && (!(*wsk2)))
			{
				if(*wsk1>tempvalue) tempvalue=*wsk1; //wykrycie maksymalnej wartosci
			}
		}
		tempvalue=tempvalue+150; //uwzglednienie maksymalnego spadku na driverze 0.9V

		if(valadc<tempvalue)
		{
			comp=tempvalue-valadc;
			if(comp>50) valueofdac+=20; else valueofdac++;
			if(valueofdac>0xfff) valueofdac=0xfff;
		}
		else if(tempvalue<valadc)
		{
			comp=valadc-tempvalue;
			if(comp>50) valueofdac-=20; else valueofdac--;
			if(valueofdac<0x0) valueofdac=0x0;

		}

		DAC_SetChannel1Data(DAC_Align_12b_R,(uint16_t)valueofdac); //aktywowanie funkcji dac
		timerregdac=1000; //zmiana wartosci co 100ms
	}
}

//Funkcja obliczajaca wartosc srednia z przetwornica ADC
uint16_t avadcfunction(uint16_t actualvalue)
{
	static uint8_t k=0;
	static uint16_t table[3];
	static uint32_t avg=0;
	uint32_t pomoc;

	if(!timadc)
	{
		table[k]=actualvalue;
		k++;
		if(k>2)
			{
			avg=table[0]+table[1]+table[2];
			avg=avg/3;
			k=0;

			pomoc=avg*3000;
			pomoc=pomoc/4096;
			pomoc=pomoc*divider;
			pomoc=pomoc/10;
			avg=pomoc;
			}
		timadc=300; //ustawienie 30ms do nastepnego pomiaru
	}

	return (uint16_t)avg;
}

//Programowanie TPS929120 EEPROM
void EEPchangeTPS(void)
{
	for(uint8_t i=0; i<6; i++)
	{
		TPS_Return_FS(i);
		ProgramEEPforFS(i);
		TPS_Force_FS(i); //przejscie do stanu FS
	}
}

//Funkcja ktora powoduje powrot ze stanu Fail State
void returnfromFS(void)
{
	for(uint8_t k=0; k<6; k++)
		{
			TPS_Return_FS(k);
	    }
}

//Funkcja sprawdzjaca czy ktoryz z tps nie znajduje sie w stanie FS jezeli tak to nalezy powrcic do normalnego stanu
void checkFSandreturn(void)
{
	static uint8_t counter=0;
	uint8_t temp;

	if(!timercheck)
	{
	temp=Read_data_TPS(counter, 0x70);
	if(temp & 0x20) TPS_Return_FS(counter);
	counter++;
	if(counter>5) counter=0;
	timercheck=6000;
	}
}

//Funkcja migajaca dioda ledinfo informujaca o programowaniu pamieci flash
void signalprogramLED(uint16_t time)
{
	static uint8_t flag=1;

	for(uint8_t i=0; i<6; i++)
	{
		if(flag)
		{
			flag=0;
			infoOFF;
		}
		else
		{
			flag=1;
			infoON;
		}
		timprog1=time*10;
		while(timprog1);
	}
}

//------------------------------------------------------------------------------------------------------------------------------------------------------
//Funkcje sluzace do obslugi pamieci flash
//Wpisanie do pamieci FLASH wartosci
void Flash_program(void)
{
  volatile uint32_t  j;

	//Odblokowanie flash
	FLASH_Unlock();
	NbrOfPage = (BANK1_WRITE_END_ADDR - BANK1_WRITE_START_ADDR)/FLASH_PAGE_SIZE; // FLASH_PAGE_SIZE;
	FLASH_ClearFlag(FLASH_FLAG_EOP|FLASH_FLAG_PGERR|FLASH_FLAG_WRPRTERR);


  // Wykasowanie pamieci
  for(EraseCounter = 0; (EraseCounter < NbrOfPage) && (FLASHStatus == FLASH_COMPLETE); EraseCounter++)
  {
    FLASHStatus = FLASH_ErasePage(BANK1_WRITE_START_ADDR + (FLASH_PAGE_SIZE * EraseCounter));
  }

		FLASH_ProgramWord(BaseAdresFlash + OffsetNazwy, 0x234fab21); //wpisanie wartosci flagi aby przy ponownym uruchamianiu oszczedzac pamiec flash
		j=(BaseAdresFlash+OffsetNazwy+4);


			 while(FLASH_COMPLETE!=FLASH_ProgramWord(j, (uint32_t)animationpwrled)){}

				j=j+4;
			while(FLASH_COMPLETE!=FLASH_ProgramWord(j, (uint32_t)powerledpower)){}

				j=j+4;
			while(FLASH_COMPLETE!=FLASH_ProgramWord(j, (uint32_t)animationstairs)){}

				j=j+4;
		    while(FLASH_COMPLETE!=FLASH_ProgramWord(j, (uint32_t)amoundofstairs)){}

				j=j+4;
		   while(FLASH_COMPLETE!=FLASH_ProgramWord(j, (uint32_t)red)){}

			   j=j+4;
		  while(FLASH_COMPLETE!=FLASH_ProgramWord(j, (uint32_t)blue)){}

			   j=j+4;
		 while(FLASH_COMPLETE!=FLASH_ProgramWord(j, (uint32_t)green)){}

			  j=j+4;

	      while(FLASH_COMPLETE!=FLASH_ProgramWord(j, (uint32_t)tresh)){}

			  			  j=j+4;

	      while(FLASH_COMPLETE!=FLASH_ProgramWord(j, (uint32_t)addtime)){}

			  			 j=j+4;

	FLASH_Lock();  //zablokowanie dostepu do pamieci flash
}

//Odczyt nastawionych parametrow z pamieci flash
void Flash_read(void)
{
	  uint32_t *p;

	  p=(uint32_t *)(BaseAdresFlash+OffsetNazwy+4);

	  animationpwrled=(uint8_t)*p; //przepisanie wartosci z flash
	  if(animationpwrled>4) animationpwrled=4;
	  p++;         //inkrementacja adresu o 4bajty gdyz mamy zmienna 4bajtowa-32bity
	  powerledpower=(uint8_t)*p; //przepisanie wartosci z flash
	  if(powerledpower>100) powerledpower=100;
	  p++;         //inkrementacja adresu o 4bajty gdyz mamy zmienna 4bajtowa-32bity
	  animationstairs=(uint8_t)*p; //przepisanie wartosci z flash
	  if(animationstairs>5) animationstairs=5;
      p++;         //inkrementacja adresu o 4bajty gdyz mamy zmienna 4bajtowa-32bity
      amoundofstairs=(uint8_t)*p; //przepisanie wartosci z flash
      if(amoundofstairs>24) amoundofstairs=24;
      p++;         //inkrementacja adresu o 4bajty gdyz mamy zmienna 4bajtowa-32bity
      red=(uint8_t)*p; //przepisanie wartosci z flash
      p++;         //inkrementacja adresu o 4bajty gdyz mamy zmienna 4bajtowa-32bity
      blue=(uint8_t)*p; //przepisanie wartosci z flash
      p++;         //inkrementacja adresu o 4bajty gdyz mamy zmienna 4bajtowa-32bity
      green=(uint8_t)*p; //przepisanie wartosci z flash
      p++;         //inkrementacja adresu o 4bajty gdyz mamy zmienna 4bajtowa-32bity
      tresh=(uint8_t)*p; //przepisanie wartosci z flash
      p++;         //inkrementacja adresu o 4bajty gdyz mamy zmienna 4bajtowa-32bity
      addtime=(uint8_t)*p; //przepisanie wartosci z flash
      p++;         //inkrementacja adresu o 4bajty gdyz mamy zmienna 4bajtowa-32bity
}

//Funkcja sprawdzajaca zmiany, ktore nastapily i jesli cos z ustawien sie zmienilo to zapisanie ustawien do pamieci flash
void checkchanges(void)
{
	static uint8_t firstuse=1;
	static uint8_t tempanimationpwrled=0, temppowerledpower=0, tempanimationstairs=0, tempamoundofstairs=0, tempred=0xff, tempblue=0xff, tempgreen=0xff;
	static uint8_t flagatoprogram=0;

	if(firstuse) //jednorazowe sprawdzenie
	{
		firstuse=0;
		tempanimationpwrled=animationpwrled;
		temppowerledpower=powerledpower;
		tempanimationstairs=animationstairs;
		tempamoundofstairs=amoundofstairs;
		tempred=red;
		tempblue=blue;
		tempgreen=green;
	}
	if(tempanimationpwrled!=animationpwrled)
	{
		tempanimationpwrled=animationpwrled;
		flagatoprogram=1;
		timerprog2=100000;
	}
	if(temppowerledpower!=powerledpower)
	{
		temppowerledpower=powerledpower;
		flagatoprogram=1;
		timerprog2=100000;
	}
	if(tempanimationstairs!=animationstairs)
	{
		tempanimationstairs=animationstairs;
		flagatoprogram=1;
		timerprog2=100000;
	}
	if(tempamoundofstairs!=amoundofstairs)
	{
		tempamoundofstairs=amoundofstairs;
		flagatoprogram=1;
		timerprog2=100000;
	}
	if(tempred!=red)
	{
		tempred=red;
		flagatoprogram=1;
		timerprog2=100000;
	}
	if(tempblue!=blue)
	{
		tempblue=blue;
		flagatoprogram=1;
		timerprog2=100000;
	}
	if(tempgreen!=green)
	{
		tempgreen=green;
		flagatoprogram=1;
		timerprog2=100000;
	}

	if(!timerprog2 && flagatoprogram)
	{
		clearwdt; //wykasowanie watchdog systemowego
		flagatoprogram=0;
		returnfromFS();
		Flash_program();
		returnfromFS();
	}

	if(flagatoprogram) infoON; else infoOFF; //zalaczenie lub wylaczenie diody LED

}
