/**
  ******************************************************************************
  * @file    stm32f1xx_it.c 
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    11-February-2014
  * @brief   Main Interrupt Service Routines.
  *          This file provides template for all exceptions handler and 
  *          peripherals interrupt service routine.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2014 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm32f1xx_it.h"

/** @addtogroup IO_Toggle
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/******************************************************************************/
/*            Cortex-M Processor Exceptions Handlers                          */
/******************************************************************************/

/**
  * @brief  This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Memory Manage exception.
  * @param  None
  * @retval None
  */
void MemManage_Handler(void)
{
  /* Go to infinite loop when Memory Manage exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Bus Fault exception.
  * @param  None
  * @retval None
  */
void BusFault_Handler(void)
{
  /* Go to infinite loop when Bus Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Usage Fault exception.
  * @param  None
  * @retval None
  */
void UsageFault_Handler(void)
{
  /* Go to infinite loop when Usage Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles Debug Monitor exception.
  * @param  None
  * @retval None
  */
void DebugMon_Handler(void)
{
}

/**
  * @brief  This function handles PendSVC exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */

#define extwdoff  GPIOC->BRR=GPIO_Pin_2  //obsluga portu watchdog
#define extwdon GPIOC->BSRR=GPIO_Pin_2


extern volatile uint32_t TIMERUS, tpstimer, timerbutton1, timerbutton2, timeranim1, timeranim2, timeranim3, timeranim4, timeranim5, timerdelay, timerwdttps, timeranim6, timeranim7,
						 timercolor, timertt, timerbt, diagtime1, diagtime2, diagtime3, timadc, timerregdac, timerclear, timercheck, timprog1, timerprog2;
extern volatile uint8_t flagawd;

void SysTick_Handler(void)
{
	if(TIMERUS) TIMERUS--;
	if(tpstimer) tpstimer--;
	if(timerbutton1) timerbutton1--;
	if(timerbutton2) timerbutton2--;
	if(timeranim1) timeranim1--;
	if(timeranim2) timeranim2--;
	if(timeranim3) timeranim3--;
	if(timeranim4) timeranim4--;
	if(timeranim5) timeranim5--;
	if(timerwdttps) timerwdttps--;
	if(timeranim6) timeranim6--;
	if(timeranim7) timeranim7--;
	if(timercolor) timercolor--;
	if(timertt) timertt--;
	if(timerbt) timerbt--;
	if(diagtime1) diagtime1--;
	if(diagtime2) diagtime2--;
	if(diagtime3) diagtime3--;
	if(timadc) timadc--;
	if(timerregdac) timerregdac--;
	if(timerclear) timerclear--;
	if(timercheck) timercheck--;
	if(timprog1) timprog1--;
	if(timerprog2) timerprog2--;


	//Obsluga watchdoga sprzetowego, ktory inicuje przejscie swiecenie do trybu fail state przy jego braku
	if(flagawd)
	{
		flagawd=0;
		extwdon;
	}
	else
	{
		flagawd=1;
		extwdoff;
	}

}


extern volatile uint8_t bufreceive[3], cRX;

//obsluga przerwan z Uart obslugujacego tps
void USART1_IRQHandler(void)
{
  if(USART_GetITStatus(USART1, USART_IT_RXNE) != RESET)
  {
	  bufreceive[cRX]=USART_ReceiveData(USART1);
	  cRX++;
	  if(cRX>2) cRX=2;
  }
}


extern volatile uint16_t uart2rx;
extern volatile unsigned char btbuff[300];

//obsluga przerwan uart obslugujacego bluetooth
void USART3_IRQHandler(void)
{
  if(USART_GetITStatus(USART3, USART_IT_RXNE) != RESET)
  {
	  btbuff[uart2rx]=USART_ReceiveData(USART3);
	  uart2rx++;
	  if(uart2rx>299) uart2rx=0;
  }
}


/******************************************************************************/
/*                 STM32F1xx Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_stm32f10x_md.s).                                            */
/******************************************************************************/

/**
  * @brief  This function handles PPP interrupt request.
  * @param  None
  * @retval None
  */
/*void PPP_IRQHandler(void)
{
}*/

/**
  * @}
  */ 


/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
